# PlayToolsLab.py
# skeleton by cs-1110 profs
# April 2016

""" Classes and functions that are useful for doing
    computations with Shakespearean text.

    The folder ThePlays must be in the current working directory.
"""
from GetData import *
import os # Needed because Windows v. non-windows use "\" v. "/" for subdirectory
          # reference, and the play files are in a subdirectory (ThePlays)


class Speech(object):
    """
    attributes:
        theSpeaker       the name of the speaker [str]
        lines            each item is a (file) line in the speech [list of str]
    """
    # Google python style: __init__ may be documented in either the class-level
    # docstring or as a docstring for __init__ itself, but the two should
    # not be mixed. http://sphinxcontrib-napoleon.readthedocs.org/en/latest/example_google.html

    def __init__(self,theSpeaker,lines):
        self.theSpeaker  = theSpeaker
        self.lines       = lines

    def __str__(self):
        """ Returns a string that is a pretty-printing of a Speech object:
        The speaker's name is on the first line.
        The following lines are indented lines from the speech.

        To pretty print a Speech object S, just write
                             print S
        """
        s = self.theSpeaker + '\n'
        for x in self.lines:
            s = s + '  ' + x + '\n'
        return s

    #################
    # Here are the three methods to be implemented...

    def Condense(self):
        """ Modifies self.lines so that it includes just the
        first line in the spoken text.
        """
        pass

    def PrettyFormat(self):
        """ Modifies self.theSpeaker so that all characters
        except the first one are lower case.
        """
        pass

    def NumQuestions(self):
        """ Returns the total number of question marks that
        appear throughout self.lines.
        """
        pass



class Scene(object):
    """
    attributes:
        actNumber       the number of the act [int]
        sceneNumber     the number of the scene [int]
        location     the location of the scene [str]
    """
    # Google python style: __init__ may be documented in either the class-level
    # docstring or as a docstring for __init__ itself, but the two should
    # not be mixed. http://sphinxcontrib-napoleon.readthedocs.org/en/latest/example_google.html

    def __init__(self,actNumber,sceneNumber,location):
        self.actNumber    = actNumber
        self.sceneNumber  = sceneNumber
        self.location     = location

    def __str__(self):
        """ Returns a string that is a pretty-printing of a Scene object:
        Example: 'Act 3  Scene 2 A public place'

        To apply this method to a scene S, write
                        print S
        """

        return 'Act %1d  Scene %1d  %s' % (self.actNumber,
                                           self.sceneNumber,
                                           self.location)


def theComedies():
    """
    Returns a list of strings that collectively name all the comedies.
    """
    return  ['ALLS WELL THAT ENDS WELL', 'AS YOU LIKE IT',
            'COMEDY OF ERRORS','CYMBELINE','LOVES LABOUR LOST',
            'MEASURE FOR MEASURE','MERCHANT OF VENICE',
            'MERRY WIVES OF WINDSOR','MIDSUMMERS NIGHTS DREAM',
            'MUCH ADO ABOUT NOTHING','PERICLES','TAMING OF THE SHREW',
            'TEMPEST','TROILUS AND CRESSIDA','TWELFTH NIGHT',
            'TWO GENTLEMAN OF VERONA','WINTERS TALE']


def theTragedies():
    """
    Returns a list of strings that collectively name all the tragedies.
    """
    return  ['ANTONY AND CLEOPATRA','CORIOLANUS', 'HAMLET',
            'JULIUS CAESAR','KING LEAR','MACBETH','OTHELLO',
             'ROMEO AND JULIET','TIMON OF ATHENS', 'TITUS ANDRONICUS']


def theHistories():
    """
    Returns a list of strings that collectively name all the histories.
    """
    return  ['HENRY IV PART 1','HENRY IV PART 2','HENRY V',
            'HENRY VI PART 1','HENRY VI PART 2', 'HENRY VI PART 3',
            'HENRY VIII','KING JOHN','RICHARD II', 'RICHARD III']


def ListOfSpeeches(p):
    """
    Returns a list of references to Speech objects. Collectively, these objects
    encode "all" the speeches in the play named by p.

    ("all" is in quotes because this simple code does miss some irregularities.
        Example: it omits portions of speeches following a blank line within a speech
        and so misses the "the play's the thing" passage in HAMLET.)

    PreC: p is an all-caps string that names a Shakespearean play in
        directory ThePlays. (No suffixes, e.g., 'HAMLET', not 'HAMLET.txt'.)
        The folder ThePlays must be in the current working directory.
    """

    L = fileToStringList(os.path.join('ThePlays', p+'.txt'))
    n = len(L)
    k = 0
    # the list of speeches will be built up in X...
    X = []
    while k<n:
        w = L[k].split()
        if len(w)==0 or L[k][0]==' ':
            # Skip over any line that begins with a blank that is not part of a speech...
            # (example: stage directions, as in the famous "Exit, pursued by a bear"
            #  in _The Winter's Tale_.)
            k+=1
        elif w[0] == 'ACT' or w[0] =='SCENE':
            # Skip over new act and new scene lines...
            k+=1
        else:
            # A speech is starting...
            theSpeaker = L[k]
            k+=2
            lines = []
            while k<n and len(L[k].split())>0:
                lines.append(L[k])
                k+=1
            X.append(Speech(theSpeaker,lines))
    return X


def ListOfScenes(p):
    """ Returns a list of references to Scene objects. Collectively, these objects
    encode all the act/scene information in the play named by p.

    If there is more than one piece of locale information given
    (example: ALLS WELL THAT ENDS WELL I.1: "Rousillon. The COUNT's palace"),
    only the portion before the first paragraph will be entered as the
    location.

     PreC: p is an all-caps string that names a Shakespearean play in
        directory ThePlays. (No suffixes, e.g., 'HAMLET', not 'HAMLET.txt'.)
        The folder ThePlays must be in the current working directory.
    """
    L = fileToStringList(os.path.join('ThePlays',p+'.txt'))
    n = len(L)
    k = 0
    actNumber = 0
    # The list of scenes will be built up in X...
    X = []
    while k<n:
        w = L[k].split()
        if len(w)==0 or L[k][0]==' ':
            # Skip over a blank line or any line that begins with a blank
            k+=1
        elif w[0] == 'ACT':
            # A new act is starting
            actNumber +=1
            sceneNumber = 0
            k+=1
        elif w[0] == 'SCENE':
            # a new scene is starting
            sceneNumber +=1
            z = L[k].split('.')
            X.append(Scene(actNumber,sceneNumber,z[1]))
            k+=1
        else:
            k+=1
    return X


def test_path_stuff():
    """Can be used to check for directory issues.
    Should be used only if advised to by a course staff member."""

    print "Debug: PlayTools thinks the current working directory is", os.getcwd()
